/* Sample.java
Oct 2002 by John Green

Simple sample of a Java method dispatcher to be called via
the jpjvm DLL / shared object.

Copyright (c) 2002 Joanju Limited. All rights reserved.
*/

import java.lang.*;
import java.io.*;
import javax.swing.*;       
import java.awt.event.*;


/**
 * Sample dispatcher class, for calling from jpjvm.
 * Some of these functions are *required* in your own dispatcher;
 * see the notes for each of the functions below.
 */
public class Sample {


	private static Sample theInstance;
	private Sample() {}
	/**
	 * Required: a public static function which returns an object
	 * instance of your dispatcher. For our Sample, we use the "Singleton"
	 * design pattern with getInstance() for returning the instance.
	 */
	public static Sample getInstance() {
		if (theInstance == null)
			theInstance = new Sample();
		return theInstance;
	}



	/**
	 * This function is required by jpjvm.dll, and it must be static.
	 * Extract a meaningful error message out of an exception.
	 * This is kind of a callback function - it gets called by jpjvm.dll
	 * if it detects that the VM has thrown an exception.
	 * You can modify this to format the message returned in any way you like.
	 */
	public static String getExceptionText(Throwable e) {
		StringWriter sw = new StringWriter();
		PrintWriter pw = new PrintWriter(sw);
		e.printStackTrace(pw);
	    return sw.toString();
	}



	/**
	 * A dispatch method. For your own dispatcher: this method is required.
	 */
	public int dispatch5i (int a1, int a2, int a3, int a4, int a5) {
		switch (a1) {
		case 1:
			return a2 + a3 + a4 + a5;
		case 2:
			helloTest();
			return 0;
		default:
			throw new RuntimeException("Invalid dispatch number: " + Integer.toString(a1));
		} // switch(a1)
	} // dispatch5i

	/**
	 * A dispatch method. For your own dispatcher: this method is required.
	 */
	public int dispatchGi (int a1, int a2, int a3, String a4, String a5) {
		switch (a1) {
		case 3:
			displayString(a4);
			return 0;
		default:
			throw new RuntimeException("Invalid dispatch number: " + Integer.toString(a1));
		} // switch(a1)
	} // dispatchGi

	/**
	 * A dispatch method. For your own dispatcher: this method is required.
	 */
	public String dispatchGs (int a1, int a2, int a3, String a4, String a5) {
		switch (a1) {
		case 4:
			return "Hello from Sample.dispatchGs!";
		default:
			throw new RuntimeException("Invalid dispatch number: " + Integer.toString(a1));
		} // switch(a1)
	} // dispatchGs




	//// ----- Just methods for demonstration purposes, from here down. ----- ////
	//// None of the following are required for your own dispatcher class.


	//// Here is a dialog box to show Java being called via jpjvm.

	private int helloTestCalls;

	private void helloTest() {
		helloTestCalls++;
		JFrame frame = new JFrame("Joanju JVM Loader Example");
		final JLabel label = new JLabel("What's up, doc?");
		frame.getContentPane().add(label);
		frame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		frame.pack();
		String theMessage = "Hi from Java! You ran me this many times: ";
		theMessage = theMessage + Integer.toString(helloTestCalls);
		JOptionPane.showMessageDialog(frame, theMessage);
		frame.dispose();
		return;
	} // helloTest()


	//// Here is a dialog box to show a string passed in as an argument.
	private void displayString(String theString) {
		JFrame frame = new JFrame("Joanju JVM Loader Example");
		final JLabel label = new JLabel("What's up, doc?");
		frame.getContentPane().add(label);
		frame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		frame.pack();
		JOptionPane.showMessageDialog(frame, theString);
		frame.dispose();
		return;
	} // displayString()


} // class Sample

