/* jpjvm.cpp

October 2002 by John Green

Notes
  - Not yet built/tested on Linux or Unix.

 
Copyright (c) 2002 Joanju Limited. All rights reserved.

*/


#include <jni.h>
#include <iostream>


#ifdef MAKE_UNIX
	#include <dlfcn.h>
	#define DLSYM dlsym
	#define DLLEXPORT
	#define PATH_SEPARATOR ':'
	#define HMODULE void *
#else	
	#include <windows.h>
	#define DLSYM GetProcAddress
	#define DLLEXPORT __declspec(dllexport)
	#define PATH_SEPARATOR ';'
#endif



namespace joanju { namespace jvmLoader {

	static char * p_outCharErr = 0;
	static char * p_outCharRetval = 0;
	static HMODULE handle = 0;
	static jclass theClass = 0;
	static jobject theObject = 0;
	static JNIEnv * env = NULL;
	static jmethodID getExceptionText = 0;
	static jmethodID dispatch5i = 0;
	static jmethodID dispatchGi = 0;
	static jmethodID dispatchGs = 0;
	static JavaVM * jvm = NULL;

	bool checkForException();
	const bool jvmIsLoaded();
	const char * outCharErr(const std::string theString);
	const char * outCharRetval(const std::string theString);

	extern "C" {
		DLLEXPORT const char * getLastMessage();
		DLLEXPORT const int javaDispatch5i(const int, const int, const int, const int, const int);
		DLLEXPORT const int javaDispatchGi(const int, const int, const int, const char *, const char *);
		DLLEXPORT const char * javaDispatchGs(const int, const int, const int, const char *, const char *);
		DLLEXPORT const int jvmtest();
		DLLEXPORT const int loadDispatcher(const char * className, const int force);
		DLLEXPORT const char * loadJVM(const int version, const char * vmlib, const char * classpath);
	} 

} } // namespaces



bool joanju::jvmLoader::checkForException() {
	jthrowable e = env->ExceptionOccurred();
	if (!e)
		return false;
	env->ExceptionDescribe();
	env->ExceptionClear();
	jstring errmsg = (jstring) env->CallStaticObjectMethod(theClass, getExceptionText, e);
	jthrowable e2 = env->ExceptionOccurred();
	if (e2) {
		env->ExceptionClear();
		outCharErr("Exception occurred calling getExceptionText");
		return true;
	}
	const char * ps = env->GetStringUTFChars(errmsg, 0);
	if (ps != 0)
		outCharErr(ps);
	env->ReleaseStringUTFChars(errmsg, ps);
	return true;
} // checkForException()



const char * joanju::jvmLoader::getLastMessage() {
	if (p_outCharErr)
		return p_outCharErr;
	else
		return outCharErr("");
} // getLastMessage()



const bool joanju::jvmLoader::jvmIsLoaded() {
	if (!handle) {
		outCharErr("JVM is not loaded");
		return false;
	} else {
		return true;
	}
} // jvmIsLoaded()



const int joanju::jvmLoader::javaDispatch5i(const int a1, const int a2, const int a3, const int a4, const int a5) {
// Returns -1 if it fails to call the method, but that's only meaningful
// if the expected result is positive. Error status should be checked otherwise.
	if (!jvmIsLoaded())
		return -1;
	if (dispatch5i==0) {
		outCharErr("Dispatcher method is not loaded");
		return -1;
	}
	int theReturn = env->CallIntMethod(theObject, dispatch5i, a1, a2, a3, a4, a5);
	if (checkForException())
		return -1;
	return theReturn; 
} // javaDispatch5i



const int joanju::jvmLoader::javaDispatchGi(const int a1, const int a2, const int a3, const char * a4, const char * a5) {
// Returns -1 if it fails to call the method, but that's only meaningful
// if the expected result is positive. Error status should be checked otherwise.
	if (!jvmIsLoaded())
		return -1;
	if (dispatchGi==0) {
		outCharErr("Dispatcher method dispatchGi is not loaded");
		return -1;
	}
	int theRet = env->CallIntMethod(theObject, dispatchGi, a1, a2, a3, env->NewStringUTF(a4), env->NewStringUTF(a5));
	if (checkForException())
		return -1;
	return theRet;
} // javaDispatch5i



const char * joanju::jvmLoader::javaDispatchGs(const int a1, const int a2, const int a3, const char * a4, const char * a5) {
// Returns blank on fail. Should check error message.
	if (!jvmIsLoaded())
		return outCharRetval("");
	if (!dispatchGs) {
		outCharErr("Dispatcher method dispatchGs is not loaded");
		return outCharRetval("");
	}
	jstring s = (jstring) env->CallObjectMethod(theObject, dispatchGs, a1, a2, a3, env->NewStringUTF(a4), env->NewStringUTF(a5));
	if (checkForException())
		return outCharRetval("");
	const char * ps = env->GetStringUTFChars(s, 0);
	if (ps != 0)
		outCharRetval(ps);
	else
		outCharRetval("");
	env->ReleaseStringUTFChars(s, ps);
	return p_outCharRetval;
} // javaDispatch5i



const int joanju::jvmLoader::loadDispatcher(const char * className, const int force) {
	// Don't reload unless we've been asked to force the issue
	if (dispatch5i && !force)
		return 0;
	// Find the class that we need to use
	theClass = env->FindClass(className);
	if (! theClass) {
		outCharErr("Could not find the dispatcher class");
		return -1;
	}
	std::string getInstSignature("()L");
	getInstSignature += className;
	getInstSignature += ";";
	jmethodID getID = env->GetStaticMethodID(theClass, "getInstance", getInstSignature.c_str());
	if (! getID) {
		outCharErr("Could not get ID to class method: getInstance");
		return -1;
	}
	theObject = env->NewObject(theClass, getID);
	theObject = env->NewGlobalRef(theObject);
	if (! theClass) {
		outCharErr("Could not create global object reference");
		return -1;
	}
	dispatch5i = env->GetMethodID(theClass, "dispatch5i", "(IIIII)I");
	if (dispatch5i == 0) {
		outCharErr("Could not get dispatcher method: dispatch5i");
		return -1;
	}
	dispatchGi = env->GetMethodID(theClass, "dispatchGi", "(IIILjava/lang/String;Ljava/lang/String;)I");
	if (dispatchGi == 0) {
		outCharErr("Could not get dispatcher method: dispatchGi");
		return -1;
	}
	dispatchGs = env->GetMethodID(theClass, "dispatchGs", "(IIILjava/lang/String;Ljava/lang/String;)Ljava/lang/String;");
	if (dispatchGs == 0) {
		outCharErr("Could not get dispatcher method: dispatchGs");
		return -1;
	}
	getExceptionText = env->GetStaticMethodID(theClass, "getExceptionText", "(Ljava/lang/Throwable;)Ljava/lang/String;");
	if (getExceptionText == 0) {
		outCharErr("Could not get dispatcher method: getExceptionText");
		return -1;
	}
	return 0;
} // loadDispatcher()



const char * joanju::jvmLoader::loadJVM(const int version, const char * vmlib, const char * classpath) {

	// Simply return if the JVM is already loaded.
	if (handle != 0)
		return outCharErr("");

	// Create a new classpath char* argument to hand off to the VM args structure
	std::string classpathString("-Djava.class.path=");
	classpathString += classpath;
	char * p_classpath = new char[classpathString.size() + 1];
	std::strcpy(p_classpath, classpathString.c_str());

	// Create our VM arguments structure
	JavaVMInitArgs vm_args;
	vm_args.nOptions = 1; // Number of options?
	JavaVMOption options[1];
	options[0].optionString = p_classpath; 
	vm_args.options = options;
	vm_args.ignoreUnrecognized = JNI_FALSE;
	vm_args.version = 0;

	// Check the VM version argument
	switch (version) {
// We don't support 1.1 right now... can be added if requested
//	case 11:
//		vm_args.version = JNI_VERSION_1_1;
//		break;
	case 12:
		vm_args.version = JNI_VERSION_1_2;
		break;
	case 13:
		vm_args.version = JNI_VERSION_1_2; // there's no JNI_VERSION_1_3
		break;
	case 14:
		vm_args.version = JNI_VERSION_1_4;
		break;
	}
	if (vm_args.version == 0)
		return outCharErr("Invalid JVM version. Valid options: 12, 13, or 14");

	// Load the library
	#ifdef MAKE_UNIX
		handle = dlopen(vmlib, RTLD_NOW);
	#else
		handle = LoadLibrary(vmlib);
	#endif
	if(handle == NULL)
		return outCharErr("Unable to load the JVM library");

	// Pointers to required functions
	typedef jint (*P_JNI_GetDefaultJavaVMInitArgs)(void *args);
	typedef jint (*P_JNI_CreateJavaVM)(JavaVM **pvm, JNIEnv ** penv, void *args);
	P_JNI_GetDefaultJavaVMInitArgs pfnGetDefaultJavaVMInitArgs = NULL;
	P_JNI_CreateJavaVM pfnCreateJavaVM = NULL;

	// Store the function pointer for getting default arguments
	pfnGetDefaultJavaVMInitArgs = (P_JNI_GetDefaultJavaVMInitArgs) DLSYM(handle, "JNI_GetDefaultJavaVMInitArgs");
	
	// Get the default arguments
	if(pfnGetDefaultJavaVMInitArgs != NULL)
		(*pfnGetDefaultJavaVMInitArgs)(&vm_args);

	// Store the function pointer for creating the VM
	pfnCreateJavaVM = (P_JNI_CreateJavaVM) DLSYM(handle, "JNI_CreateJavaVM");
	if (!pfnCreateJavaVM)
		return outCharErr("Can't get Proc Address");

	// Create the Java VM
	jint res = -1;
	if(pfnCreateJavaVM != NULL)
		res = (*pfnCreateJavaVM)(&jvm,&env,&vm_args);
	if (res < 0)
		return outCharErr("Can't create Java VM");

	return outCharErr("");

} // loadJVM()



const char * joanju::jvmLoader::outCharErr(const std::string theString) {
// We manage the memory for the passed-back pointer-to-char-string.
// As far as PROGRESS is concerned, the pointer is only "good" up until
// the next function call.
	delete [] p_outCharErr;
	p_outCharErr = new char[theString.size() + 1];
	std::strcpy(p_outCharErr, theString.c_str());
	return p_outCharErr;
} // outCharErr()



const char * joanju::jvmLoader::outCharRetval(const std::string theString) {
// Same as outCharErr, but p_outCharErr is used for error/status messages.
// This p_outCharRetval is used for function return values.
	delete [] p_outCharRetval;
	p_outCharRetval = new char[theString.size() + 1];
	std::strcpy(p_outCharRetval, theString.c_str());
	return p_outCharRetval;
} // outCharRetval()




