/* jpjvm/test.p
 * October 2002 by John Green
 *
 * Example code for calling a Java dispatcher via our
 * JVM wrapper "jpjvm.dll".
 *
 * IMPORTANT: You must configure the JVM location and
 *            and version, below.
 *
 * Copyright (c) 2002 John Green. All rights reserved.
 */


/* --- jvm settings ---
 * YOU MUST SET THESE!
 * See the notes in "PROCEDURE loadJVM".
 * --------------- */
DEFINE VARIABLE jvmPath AS CHARACTER NO-UNDO
   INITIAL "/progfile/jdk/jre/bin/client/jvm.dll".
DEFINE VARIABLE jvmVersion AS INTEGER NO-UNDO
   INITIAL 14. /* java version 1.4 */



DEFINE VARIABLE tempInt AS INTEGER NO-UNDO.
DEFINE VARIABLE tempPtr AS MEMPTR NO-UNDO.
DEFINE VARIABLE tempChar AS CHARACTER NO-UNDO.


/* --- checkForError ---
 * Handy function for checking if there's an
 * error message waiting for us.
 * --------------------- */
FUNCTION checkForError RETURNS LOGICAL:
  DEFINE VARIABLE thePtr AS MEMPTR NO-UNDO.
  RUN getLastMessage(OUTPUT thePtr).
  ASSIGN tempChar = GET-STRING(thePtr, 1).
  IF tempChar <> "" THEN DO:
    MESSAGE tempChar VIEW-AS ALERT-BOX TITLE "Return from loadJVM".
    RETURN TRUE.
  END.
  RETURN FALSE.
END.


/* --- Load the JVM --- */
RUN loadJVM (
    INPUT jvmVersion,
    INPUT jvmPath,
    INPUT "",  /* classpath, like java.exe -classpath */
    OUTPUT tempPtr
    ).
ASSIGN tempChar = GET-STRING(tempPtr, 1).
IF tempChar <> "" THEN DO:
  MESSAGE tempChar VIEW-AS ALERT-BOX TITLE "Return from loadJVM".
  RETURN.
END.


/* --- Load the Java dispatcher ---
 * Note that classname must be fully qualified with /
 * as the segment delimiter, ex: "com/joanju/Dispatcher"
 * -------------------------------- */
RUN loadDispatcher (
    INPUT "Sample",   /* Classname */
    INPUT 0, /* don't force reload */
    OUTPUT tempInt
    ).
IF tempInt <> 0 THEN DO:
  checkForError().
  RETURN.
END.


/* --- Example #1 from Sample Dispatcher ---
 * Prove that Java can add.
 * ----------------------------------------- */
RUN javaDispatch5i (1, 2, 3, 4, 5, OUTPUT tempInt).
MESSAGE "2+3+4+5=" tempInt VIEW-AS ALERT-BOX.
IF checkForError() THEN RETURN.


/* --- Example #2 from Sample Dispatcher ---
 * Shows a "hello from java" dialog.
 * Run this test.p program multiple times to
 * prove that Java can count.
 * ----------------------------------------- */
RUN javaDispatch5i (2, 0, 0, 0, 0, OUTPUT tempInt).
IF checkForError() THEN RETURN.


/* --- Example #3 from Sample Dispatcher ---
 * Prove that Java can listen.
 * ----------------------------------------- */
RUN javaDispatchGi (3, 0, 0, "String from 4gl!", "", OUTPUT tempInt).
IF checkForError() THEN RETURN.


/* --- Example #4 from Sample Dispatcher ---
 * Prove that Java can talk.
 * ----------------------------------------- */
RUN javaDispatchGs (4, 0, 0, "", "", OUTPUT tempPtr).
IF checkForError() THEN RETURN.
ASSIGN tempChar = GET-STRING(tempPtr, 1).
MESSAGE tempChar VIEW-AS ALERT-BOX TITLE "Java sends this message".

/* --- Demonstrate jpjvm's exception handling ---
 * We'll try calling dispatch #999, which is invalid.
 * The Java dispatcher throws an exception, which is caught.
 * That exception is then handed off to the getExceptionText()
 * method within the dispatcher, for formatting and extracting
 * a meaningful error message from the exception. The resulting
 * string is then available via getLastMessage.
 * ---------------------------------------------- */
RUN javaDispatch5i (999, 0, 0, 0, 0, OUTPUT tempInt).
MESSAGE
  "Now we expect to see the text from an exception thrown by the Java code..."
  VIEW-AS ALERT-BOX.
checkForError().



/* ------- Procedures ------- */


PROCEDURE javaDispatch5i EXTERNAL "jpjvm.dll" CDECL PERSISTENT:
  DEFINE INPUT PARAMETER a1 AS LONG.
  DEFINE INPUT PARAMETER a2 AS LONG.
  DEFINE INPUT PARAMETER a3 AS LONG.
  DEFINE INPUT PARAMETER a4 AS LONG.
  DEFINE INPUT PARAMETER a5 AS LONG.
  DEFINE RETURN PARAMETER theRet AS LONG.
END PROCEDURE.



PROCEDURE javaDispatchGi EXTERNAL "jpjvm.dll" CDECL PERSISTENT:
  DEFINE INPUT PARAMETER a1 AS LONG.
  DEFINE INPUT PARAMETER a2 AS LONG.
  DEFINE INPUT PARAMETER a3 AS LONG.
  DEFINE INPUT PARAMETER a4 AS CHARACTER.
  DEFINE INPUT PARAMETER a5 AS CHARACTER.
  DEFINE RETURN PARAMETER theRet AS LONG.
END PROCEDURE.



PROCEDURE javaDispatchGs EXTERNAL "jpjvm.dll" CDECL PERSISTENT:
  DEFINE INPUT PARAMETER a1 AS LONG.
  DEFINE INPUT PARAMETER a2 AS LONG.
  DEFINE INPUT PARAMETER a3 AS LONG.
  DEFINE INPUT PARAMETER a4 AS CHARACTER.
  DEFINE INPUT PARAMETER a5 AS CHARACTER.
  DEFINE RETURN PARAMETER theRet AS MEMPTR.
END PROCEDURE.



PROCEDURE loadDispatcher EXTERNAL "jpjvm.dll" CDECL PERSISTENT:
/* INPUT className:
 *   example: "com/joanju/Dispatcher"
 * INPUT forceReload:
 *   0 for false, 1 for true. Use 1 if you want to switch from
 *   one dispatching class to another.
 * RETURN retStatus:
 *   0 on success, -1 on fail (use getLastMessage)
 */
  DEFINE INPUT PARAMETER className AS CHARACTER.
  DEFINE INPUT PARAMETER forceReload AS LONG.
  DEFINE RETURN PARAMETER retStatus AS LONG.
END PROCEDURE.



PROCEDURE loadJVM EXTERNAL "jpjvm.dll" CDECL PERSISTENT:
/* NOTE: the character string at the returned MEMPTR
 *       is memory managed by jpjvm.dll. Do NOT free it.
 * Note: It is OK to run this again once the jvm is already
 *       loaded. It will simply return with no effect.
 * INPUT version:
 *   The version number corresponds to the Java
 *   virtual machine version number. Valid arguments
 *   are 12, 13, and 14, corresponding to Java versions
 *   1.2, 1.3, and 1.4.
 *   1.1 is not currently supported - let us know if you need
 *   it, we can easily add it.
 * INPUT vmlib:
 *   The full path to your jvm.dll, for example, this is what
 *   it looks like on my XP machine:
 *   "/progfile/jdk/jre/bin/client/jvm.dll"
 * INPUT classpath:
 *   Same as you would pass with the -classpath for java.exe.
 * OUTPUT errorMessage
 *   As mentioned above: DO NOT FREE this memptr.
 *   On successful load of the jvm, this memptr will point to
 *   an empty string "". On fail, the string will contain an
 *   error message.
 */
  DEFINE INPUT PARAMETER version AS LONG.
  DEFINE INPUT PARAMETER vmlib AS CHARACTER.
  DEFINE INPUT PARAMETER classpath AS CHARACTER.
  DEFINE RETURN PARAMETER errorMessage AS MEMPTR.
END PROCEDURE. /* loadJVM */  



PROCEDURE getLastMessage EXTERNAL "jpjvm.dll" CDECL PERSISTENT:
/* NOTE: the character string at the returned MEMPTR
 *       is memory managed by jpjvm.dll. Do NOT free it.
 */
  DEFINE RETURN PARAMETER errorMessage AS MEMPTR.
END PROCEDURE.  
